/*
 * logutil.c
 *
 *  Created on: 15/05/2013
 *      Author: Jordi Fenés
 */

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"
#include "printf.h"
#include "logutil.h"
#include "config.h"

/* Declare a variable of type xSemaphoreHandle */
static xSemaphoreHandle B_Mutex = NULL;
static xSemaphoreHandle N_Mutex = NULL;
static xSemaphoreHandle R1_Mutex = NULL;
static xSemaphoreHandle R2_Mutex = NULL;

/*****************************************************************************
** Function name:		Lock_Task
**
** Descriptions:		Use Semaphore or vTaskSuspend for block process
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void Lock_Task(uint8_t Tipus)
{
	if (LOCK == 2)
	{
		switch ( Tipus )
		{
			case SEM_R2:
				if(R2_Mutex == NULL)		R2_Mutex = xSemaphoreCreateRecursiveMutex();
				xSemaphoreTakeRecursive(R2_Mutex, portMAX_DELAY);
				break;
			case SEM_R1:
				if(R1_Mutex == NULL)		R1_Mutex = xSemaphoreCreateRecursiveMutex();
				xSemaphoreTakeRecursive(R1_Mutex, portMAX_DELAY);
				break;
			case SEM_N:
				if(N_Mutex == NULL)		N_Mutex = xSemaphoreCreateMutex();
				xSemaphoreTake(N_Mutex, portMAX_DELAY );
				break;
			case SEM_B:
				if(B_Mutex == NULL)		vSemaphoreCreateBinary(B_Mutex);
				xSemaphoreTake(B_Mutex, portMAX_DELAY );
				break;
		}
	}
	else if (LOCK == 1)
	{
		vTaskSuspendAll();
	}
}

/*****************************************************************************
** Function name:		UnLock_Task
**
** Descriptions:		Use Semaphore or vTaskResume for unblock process
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void UnLock_Task(uint8_t Tipus)
{
	if (LOCK == 2)
	{
		switch ( Tipus )
		{
			case SEM_R2:
				xSemaphoreGiveRecursive(R2_Mutex);
				break;
			case SEM_R1:
				xSemaphoreGiveRecursive(R1_Mutex);
				break;
			case SEM_N:
				xSemaphoreGive(N_Mutex);
				break;
			case SEM_B:
				xSemaphoreGive(B_Mutex);
				break;
		}
	}
	else if (LOCK == 1)
	{
		xTaskResumeAll();
	}
}

/*****************************************************************************
** Function name:		DelayMs
**
** Descriptions:		Procedure implementing an active wait based on a simple loop
**
** parameters:			tipo(Brute or Task), ms miliseconds to delay
** Returned value:		None
**
*****************************************************************************/
void Delay_Ms(uint8_t base, uint32_t ms)
{
	if (base == 0) // Active Delay
	{
		uint32_t i;
		for(i=0; i<(((uint32_t)7140)*ms);i++); // loop 7140 approximate 1 millisecond
	}
	else if (base == 1)  // Passive Delay
	{
		const portTickType xDelayMs = (portTickType) ms / portTICK_RATE_MS;
		vTaskDelay(xDelayMs);
	}
}

/*****************************************************************************
** Function name:		Log_Mode
**
** Descriptions:		Print info for Mode, __FILE__ and __LINE__
**
** parameters:			UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Log_Mode(uint8_t Mode, uint8_t Res, char *Fun)
{
	if (Mode == LOG_DBG)	Print_Text(UART_DBG, (char *)"[DBG]--> ");
	else if	(Mode == LOG_INFO)	Print_Text(UART_DBG, (char *)"[INFO]--> ");
	else if	(Mode == LOG_WARN)	Print_Text(UART_DBG, (char *)"[WARN]--> ");
	else	Print_Text(UART_DBG, (char *)"[ERR]--> ");
	Print_Text(UART_DBG, Fun);
	Print_Text(UART_DBG, (char *)" : ");
	if	(Mode == LOG_WARN && Res == FALSE)
	{
		Print_Text(UART_DBG, (char *)"AVÍS - ");
	}
	if (Res == FALSE)	Print_Text(UART_DBG, (char *)"FALLA ");

	//Print_Text(UART_DBG, (char *)__FILE__);
	//Print_TextNum(UART_DBG, (char *)" -", __LINE__);
}

/*****************************************************************************
** Function name:		Log_T_Print
**
** Descriptions:		Print a text with type of end line
**
** parameters:			Mode info, Result, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Log_T_Print(uint8_t Mode, uint8_t Res, char *Fun, char *Text)
{
	Log_Mode(Mode, Res, Fun);
	Print_T_CRLF (UART_DBG, Text);
}

/*****************************************************************************
** Function name:		Log_TT_Print
**
** Descriptions:		Print a text concatenated at Word passed as a parameter
**
** parameters:			Mode info, Result, Pointer to the text, pointer to the word
** Returned value:		None
**
*****************************************************************************/

void Log_TT_Print(uint8_t Mode, uint8_t Res, char *Fun, char *Text, char *Word)
{
	Log_Mode(Mode, Res, Fun);
	Print_Text(UART_DBG, Text);
	Print_TT_CRLF(UART_DBG, (char *)" : ", Word);
}

/*****************************************************************************
** Function name:		Log_TT_Print
**
** Descriptions:		Print a text concatenated at Num passed as a parameter
**
** parameters:			Mode info, Result, Pointer to the text, Value to print
** Returned value:		None
**
*****************************************************************************/
void Log_TN_Print(uint8_t Mode, uint8_t Res, char *Fun, char *Text, int32_t Value)
{
	Log_Mode(Mode, Res, Fun);
	Print_Text(UART_DBG, Text);
	Print_TN_CRLF(UART_DBG, (char *)" : ", Value);
}
