/*
 * printf.c
 *
 *  Created on: 20/03/2013
 *      Author: Jordi Fenés
 */

#include "LPC17xx.h"
#include "type.h"
#include "logutil.h"
#include "uart.h"
#include "printf.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

extern volatile uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
extern volatile uint8_t UART0Buffer[BUFSIZE], UART1Buffer[BUFSIZE], UART2Buffer[BUFSIZE], UART3Buffer[BUFSIZE];

/*****************************************************************************
** Function name:		Print_Flush
**
** Descriptions:		Erase the buffer used in UART port.
**
** parameters:			UART port
** Returned value:		None
**
*****************************************************************************/
void Print_Flush(uint8_t PortNum)
{
	UART_Flush(PortNum);
}


/*****************************************************************************
** Function name:		Print_Init
**
** Descriptions:		Initialize the UART Print connection
**
** parameters:			UART port, Baudrate to connect
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Print_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint8_t ret = FALSE;
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		ret = UART_Init(PortNum, Baudrate);
		UART_Flush (PortNum);
	}
	UnLock_Task(SEM_N);
	return ret;
}

/*****************************************************************************
** Function name:		Print_End
**
** Descriptions:		Close the UART Print connection
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Print_End(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		ret = UART_Close(PortNum);
		UART_Flush (PortNum);
	}
	UnLock_Task(SEM_N);
	return ret;
}

/*****************************************************************************
** Function name:		Print_Text
**
** Descriptions:		Print a text
**
** parameters:			UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Print_Text(uint8_t PortNum , char *Text)
{
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		UART_Send(PortNum, Text);
	}
	UnLock_Task(SEM_N);
}

/*****************************************************************************
** Function name:		Print_T_CRLF
**
** Descriptions:		Print a text with type of end line
**
** parameters:			UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Print_T_CRLF(uint8_t PortNum , char *Text)
{
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		UART_Send(PortNum, Text);
		UART_Send(PortNum, (char *)"\r\n");
	}
	UnLock_Task(SEM_N);
}

/*****************************************************************************
** Function name:		Print_2T_CRLF
**
** Descriptions:		Print a text with type of end line
**
** parameters:			First UART port, Second UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Print_2T_CRLF(uint8_t PortNum, uint8_t PortTerm , char *Text)
{
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		UART_Send(PortNum, Text);
		UART_Send(PortNum, (char *)"\r\n");
		UART_Send(PortTerm, Text);
		UART_Send(PortTerm, (char *)"\r\n");
	}
	UnLock_Task(SEM_N);
}

/*****************************************************************************
** Function name:		Print_TT_CRLF
**
** Descriptions:		Print a text concatenated at Word passed as a parameter
**
** parameters:			Pointer to the text, pointer to the word
** Returned value:		None
**
*****************************************************************************/

void Print_TT_CRLF(uint8_t PortNum , char *Text , char *Word)
{
	char buf[GETSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		sprintf(buf, "%s %s\r\n", Text, Word);
		UART_Send(PortNum, buf);
	}
	UnLock_Task(SEM_N);
	free(buf);
}

/*****************************************************************************
** Function name:		Print_TextNum
**
** Descriptions:		Print a text and a number passed as a parameter
**
** parameters:			Pointer to the text, number value
** Returned value:		None
**
*****************************************************************************/

void Print_TextNum(uint8_t PortNum , char *Text , int32_t Num)
{
	char buf[GETSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		sprintf(buf, "%s %i", Text, Num);
		UART_Send(PortNum, buf);
	}
	UnLock_Task(SEM_N);
	free(buf);
}

/*****************************************************************************
** Function name:		Print_TextFloat
**
** Descriptions:		Print a text and a Float number passed as a parameter
**
** parameters:			Pointer to the text, Float number value
** Returned value:		None
**
*****************************************************************************/

void Print_TextFloat(uint8_t PortNum , char *Text , float Num)
{
	char buf[GETSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		sprintf(buf, "%s %.2f", Text, Num);
		UART_Send(PortNum, buf);
	}
	UnLock_Task(SEM_N);
	free(buf);
}

/*****************************************************************************
** Function name:		Print_TN_CRLF
**
** Descriptions:		Print a text and a number with type of end line
**
** parameters:			Pointer to the text, number value, type of end line
** Returned value:		None
**
*****************************************************************************/

void Print_TN_CRLF(uint8_t PortNum , char *Text , int32_t Num)
{
	char buf[GETSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		sprintf(buf, "%s %i\r\n", Text, Num);
		UART_Send(PortNum, buf);
	}
	UnLock_Task(SEM_N);
	free(buf);
}

/*****************************************************************************
** Function name:		Print_TF_CRLF
**
** Descriptions:		Print a text and a Float number with type of end line
**
** parameters:			Pointer to the text, Float number value, type of end line
** Returned value:		None
**
*****************************************************************************/

void Print_TF_CRLF(uint8_t PortNum , char *Text , float Num)
{
	char buf[GETSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Task(SEM_N);
	{
		sprintf(buf, "%s %.2f\r\n", Text, Num);
		UART_Send(PortNum, buf);
	}
	UnLock_Task(SEM_N);
	free(buf);
}

/*****************************************************************************
** Function name:		UART_Scanf
**
** Descriptions:		Read the UART port waiting time parameter
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint32_t UART_Scanf(uint8_t PortNum, char *Text, uint32_t Time)
{
	uint32_t oldCount = 0;
	uint32_t newCount = 0;
	{
		if ( PortNum == 0 )
		{
			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART0Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART0->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART0Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 1 )
		{
			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART1Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART1->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART1Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 2 )
		{
			LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART2Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART2->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART2Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 3 )
		{
			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART3Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART3->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART3Buffer, newCount ); // Obtain UART data
		}
		UART_Flush(PortNum); // Erase and initialize UART buffer
	}
	Text[newCount] = '\0'; // add end string. Now we have a string
	#if LOG == LOG_DBG
		Print_Text(UART_DBG, Text);
	#endif
	return (newCount);
}

/*****************************************************************************
** Function name:		UART_Read
**
** Descriptions:		Read the UART port waiting time parameter
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint32_t UART_Read(uint8_t PortNum, char *Text, char *Value, uint32_t Time, uint32_t Size)
{
	uint32_t len_text = 0;
	Text[0] = '\0';
	while (Time > 0)
	{
		if ((PortNum == 0) && (UART0Count != 0))
		{
			LPC_UART0->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART0Buffer) + strlen(Text) < Size)
			{
				strncat(Text, (char*)UART0Buffer, UART0Count);
			}
			else strcpy(Text, (char *)Overflow);
			UART0Count = 0;
			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 15;
		}
		else if ((PortNum == 1) && (UART1Count != 0))
		{
			LPC_UART1->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART1Buffer) + strlen(Text) < Size)
			{
				strncat(Text, (char*)UART1Buffer, UART1Count);
			}
			else strcpy(Text, (char *)Overflow);
			UART1Count = 0;
			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 15;
		}
		else if ((PortNum == 2) && (UART2Count != 0))
		{
			LPC_UART2->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART2Buffer) + strlen(Text) < Size)
			{
				strncat(Text, (char*)UART2Buffer, UART2Count);
			}
			else strcpy(Text, (char *)Overflow);
			UART2Count = 0;
			LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 15;
		}
		else if ((PortNum == 3) && (UART3Count != 0))
		{
			LPC_UART3->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART3Buffer) + strlen(Text) < Size)
			{
				strncat(Text, (char*)UART3Buffer, UART3Count);
			}
			else strcpy(Text, (char *)Overflow);
			UART3Count = 0;
			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 15;
		}
		Delay_Ms(DELAY, 1);
		Time--;
	}
	UART_Flush(PortNum);
	len_text = strlen(Text);
#if LOG == LOG_DBG
	if (len_text > 0)	Print_Text(UART_DBG, Text);
#endif
	return len_text;
}


/*****************************************************************************
** Function name:		Check_Read
**
** Descriptions:		Read the UART port waiting time parameter and check with value
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/

uint8_t Check_Read(uint8_t PortNum, char *Text, char *Value, uint32_t Time, uint32_t Size)
{
	uint8_t ret = FALSE;
	if (UART_Read(PortNum, Text, Value, Time, Size) > 0)
	{
		if (strstr(Text, Value) != NULL) ret = TRUE;
	}
	return ret;
}
