/****************************************************************************
 *   $Id:: wdt.c 5752 2010-12-01 00:01:10Z usb00423                         $
 *   Project: NXP LPC17xx Watchdog Timer example
 *
 *   Description:
 *     This file contains WDT code example which include WDT initialization, 
 *     WDT interrupt handler, and APIs for WDT access.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#include "LPC17xx.h"
#include "type.h"
//#include "timer.h"
#include "wdt.h"

volatile uint32_t WDT_COUNT = 0;

/*****************************************************************************
** Function name:		WDT_IRQHandler
**
** Descriptions:		Watchdog timer interrupt handler
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void WDT_IRQHandler(void) 
{
  LPC_WDT->WDMOD &= ~WDTOF;		/* clear the time-out terrupt flag */
  WDT_COUNT++;
  return;
}

/*****************************************************************************
** Function name:		WDT_Init
**
** Descriptions:		Initialize watchdog timer, install the watchdog timer
** 						interrupt handler. Default to IRC Clock source
**
** parameters:			None
** Returned value:		true or false, return false if the VIC table
**						is full and WDT interrupt handler can be installed.
** 
*****************************************************************************/
uint8_t WDT_Init( void )
{
  WDT_COUNT = 0;
  NVIC_EnableIRQ(WDT_IRQn);

  LPC_WDT->WDTC = WDT_FEED_VALUE;	/* once WDEN is set, the WDT will start after feeding */
  LPC_WDT->WDMOD = WDEN;

  LPC_WDT->WDFEED = 0xAA;		/* Feeding sequence */
  LPC_WDT->WDFEED = 0x55;
  return( TRUE );
}

/*****************************************************************************
** Function name:		WDT_Start		(New Creation)
**
** Descriptions:		Start the watchdog timer
**
** parameters:			Timer to count for watchdog
** Returned value:		none
**
*****************************************************************************/
void WDT_Start(uint32_t Timeout)
{
	WDT_COUNT = 0;
	NVIC_EnableIRQ(WDT_IRQn);
	WDT_Set(Timeout);
	LPC_WDT->WDMOD = (WDEN|WDRESET);
	WDT_Feed();
	return;
}

/*****************************************************************************
** Function name:		WDT_Stop		(New Creation)
**
** Descriptions:		Stop the watchdog timer
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void WDT_Stop( void )
{
	NVIC_DisableIRQ(WDT_IRQn);
	LPC_WDT->WDMOD &= ( ~(WDEN_OFF|WDRESET_OFF) );
	return;
}

/*****************************************************************************
** Function name:		WDT_Set		(New Creation)
**
** Descriptions:		Calculate and set the Time for watchdog timer
**
** parameters:			Time to count for watchdog
** Returned value:		true or false if the Time is set.
**
*****************************************************************************/
uint8_t WDT_Set(uint32_t Timeout)
{
	uint8_t ret = FALSE;
	uint32_t pclk_wdt = 4000000, tempo = 0;
	// Calculate TC in WDT
	tempo  = (((pclk_wdt) / WDT_US_INDEX) * (Timeout / 4));
	// Check if it valid
	if ((tempo >= WDT_TIMEOUT_MIN) && (tempo <= WDT_TIMEOUT_MAX))
	{
		LPC_WDT->WDTC = tempo;
		ret = TRUE;
	}
	return ret;
}

/*****************************************************************************
** Function name:		WDT_Feed
**
** Descriptions:		Feed watchdog timer to prevent it from timeout
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void WDT_Feed( void )
{
  LPC_WDT->WDFEED = 0xAA;		/* Feeding sequence */
  LPC_WDT->WDFEED = 0x55;
  return;
}

/******************************************************************************
**                            End Of File
******************************************************************************/
