/*
 * wifly.c
 *
 *  Created on: 23/03/2013
 *      Author: Jordi Fenés
 */
#include "LPC17xx.h"
#include "type.h"
#include "logutil.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

static char Buffer[GETSIZE];
static uint32_t Estat = NORMAL;

/*****************************************************************************
** Function name:		Wifly_Sleep
**
** Descriptions:		Send 'sleep' to low power consumption
**
** parameters:			UART port to send
** Returned value:		Void
**
*****************************************************************************/
void Wifly_Sleep(uint8_t PortNum)
{
	/* Prepared to create a mutex type semaphore to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Print_Flush(PortNum);
	Print_T_CRLF(PortNum, (char *)"sleep");
	//Delay_Ms(DELAY,250);
	Estat = NORMAL;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, TRUE, (char *)"Wifly_Sleep", (char *)"POSAR A DORMIR MODUL WyFly");
#endif
	UnLock_Task(SEM_R1);
	return;
}

/*****************************************************************************
** Function name:		Wifly_WakeUp
**
** Descriptions:		Send 'sleep' to low power consumption
**
** parameters:			UART port to send
** Returned value:		Void
**
*****************************************************************************/
void Wifly_WakeUp(uint8_t PortNum)
{
	uint32_t ret = FALSE;
	/* Prepared to create a mutex type semaphore to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	ret = Wifly_Set_Check(PortNum, (char*)"Wake_Up", (char*)"Listen", 6000);
	Delay_Ms(DELAY,100);
	Estat = NORMAL;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_WakeUp", (char *)"DESPERTAR AL MODUL WyFly");
#endif
	UnLock_Task(SEM_R1);
	return;
}

/*****************************************************************************
** Function name:		Wifly_Reset
**
** Descriptions:		Hardware reset the WiFly device. PIN 6-24 (P0.22) of LPC1769
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint8_t Wifly_Reset(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	/* Enable GPIO into PCONP */
	LPC_SC->PCONP |= (1 << 15);
	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	LPC_GPIO0->FIODIR |= ( 1 << WIFLY_PIN_RST );
	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << WIFLY_PIN_RST;
	Delay_Ms(DELAY,1); // min 160us
	LPC_GPIO0->FIOSET = 1 << WIFLY_PIN_RST;
	ret = Check_Read(PortNum, (char *)Buffer, (char*)"Listen", 6000, GETSIZE);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_Reset", (char *)"RESET DEL MODUL WyFly");
#endif
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Init
**
** Descriptions:		Defines the used UART, its baudrate and reset device
** 						This operation produce a few delay.
**
** parameters:			UART port ,  desired baudrate
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	Print_Flush(PortNum);
	Buffer[0] = '\0';
	ret = Print_Init(PortNum, Baudrate);
	if (ret == TRUE)	ret = Wifly_Reset(PortNum);
	Delay_Ms(DELAY,100);
	Estat = NORMAL;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_Init", (char *)"INICIACIO MODUL WyFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_End
**
** Descriptions:		Close the UART port used for WiFly device.
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_End(uint8_t PortNum)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	{
		Print_Flush(PortNum);
		ret = Print_End(PortNum);
	}
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_End", (char *)"TANCAMENT PORT MODUL WyFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Read
**
** Descriptions:		Read the UART Buffer for received New Wifly Remote Data
**
** parameters:			UART Port and String to Check in received data
** Returned value:		TRUE or FALSE if found
**
*****************************************************************************/
uint8_t Wifly_Read(uint8_t PortNum, char *Check, uint32_t Time)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	{
		ret = Check_Read(PortNum, (char *)Buffer, Check, Time, GETSIZE);
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Set_OK
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Set_OK(uint8_t PortNum, char *Comand, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	Print_Flush(PortNum);
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *)Buffer,(char *)"AOK" , Time, GETSIZE);
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Set_TT_OK
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, char pointer to join at command
** 						and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Set_TT_OK(uint8_t PortNum, char *Comand, char *Value, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	Print_Flush(PortNum);
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *)Buffer,(char *)"AOK" , Time, GETSIZE);
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Set_TN_OK
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, number to join at command
** 						and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Set_TN_OK(uint8_t PortNum, char *Comand, int32_t Num, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	Print_Flush(PortNum);
	Print_TN_CRLF(PortNum, Comand, Num);
	ret = Check_Read(PortNum, (char *)Buffer,(char *)"AOK" , Time, GETSIZE);
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Set_Check
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, char pointer to check
** 						and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Set_Check(uint8_t PortNum, char *Comand, char *Check, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	//Buffer[0] = '\0';
	if (strlen(Comand) > 0)
	{
		Print_Flush(PortNum);
		Print_T_CRLF(PortNum, Comand);
	}
	ret = Check_Read(PortNum, (char *)Buffer, Check, Time, GETSIZE);
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Set_TT_Check
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, char pointer Value to join at command
** 						char pointer to Check, and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Set_TT_Check(uint8_t PortNum, char *Comand, char *Value, char *Check, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	//Buffer[0] = '\0';
	Print_Flush(PortNum);
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *)Buffer, Check, Time, GETSIZE);
	//UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_CMD
**
** Descriptions:		Send '$$$' to enter at Command Mode of WiFly
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_CMD(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == COMMAND) return TRUE;
	Print_Flush(PortNum);
	Print_Text(PortNum, (char *)"$$$");
	ret = Check_Read(PortNum, (char *)Buffer, (char *)"CMD", 500, GETSIZE);
	if (ret == TRUE) Estat = COMMAND;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_CMD", (char *)"ENTRADA A MODE COMANDES");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Exit
**
** Descriptions:		Send 'exit' to return at Serial Mode of WiFly
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Exit(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) ret = TRUE;
	else
	{
		ret = Wifly_Set_Check(PortNum, (char *)"exit", (char *)"EXIT", 600);
		if (ret == TRUE) Estat = NORMAL;
	}
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_Exit", (char *)"SORTIDA DE MODE COMANDES");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Open
**
** Descriptions:		Send 'open' with Address (empty or text)
**
** parameters:			UART port to send and Address to connect
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Open(uint8_t PortNum, char *Address)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Wifly_Set_OK(PortNum, (char *)"set c r 0", 600);
	Print_Flush(PortNum);
	if (strlen(Address) > 0)	Print_TT_CRLF(PortNum, (char *)"open", Address);
	else	Print_T_CRLF(PortNum, (char *)"open");
	ret = Check_Read(PortNum, (char *)Buffer, (char *)"*OPEN*", 4000, GETSIZE);
	if (ret == TRUE) Estat = NORMAL;
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_Open", (char *)"CONNEXIO TIPUS TCP/IP", Address);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Close
**
** Descriptions:		Send 'close' to close open TCP connection
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Close(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"close", (char *)"*CLOS*", 600);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Wifly_Close", (char *)"TANCADA CONNEXIO A INTERNET");
#endif
	//Wifly_Exit(PortNum);
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Save
**
** Descriptions:		Send 'save' to store actual configuration
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Save(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"save", (char *)"Storing in config", 600);
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_Save", (char *)"GUARDADA CONFIGURACIO WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Reboot
**
** Descriptions:		Send 'reboot' now initialize the WiFly
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Reboot(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char*)"reboot", (char*)"Listen", 6000);
	if (ret == TRUE) Estat = NORMAL;
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_Reboot", (char *)"REINICIACIO DEL WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Connect
**
** Descriptions:		Connect to a Wi-Fy network
**
** parameters:			Pointer to Buffer, Mode, SSID and Phrase to connect
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Connect(uint8_t PortNum, uint8_t Mode, char *ssid, char* Phrase)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_TN_OK(PortNum, (char *)"set w a", Mode, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set w s", ssid, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set w p", Phrase, 850);
	if (ret == TRUE)	ret = Wifly_Set_TT_Check(PortNum, (char *)"join", ssid, (char *)"Associated!", 8000);
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_Connect", (char *)"CONNEXIO WLAN DEL WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_AutoConnect
**
** Descriptions:		Connect to a Wi-Fy network mode Auto at Start
**
** parameters:			Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint8_t Wifly_AutoConnect(uint8_t PortNum, uint8_t Mode, char *ssid, char* Phrase)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_TN_OK(PortNum, (char *)"set w a", Mode, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set w s", ssid, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set w p", Phrase, 850);
	if (ret == TRUE)	ret = Wifly_Set_OK(PortNum, (char *)"set w j 1", 600);
	if (ret == TRUE)	ret = Wifly_Set_Check(PortNum, (char *)"join", (char *)"Associated!", 8000);
	if (ret == TRUE) ret = Wifly_Save(PortNum);
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_AutoConnect", (char *)"AUTOCONNEXIO WLAN DEL WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_FixIP
**
** Descriptions:		Connect to a Wi-Fy network mode Auto at Start
**
** parameters:			Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint8_t Wifly_FixIP(uint8_t PortNum, char *IP, char *Mask, char* GW)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_OK(PortNum, (char *)"set c r 0", 600);
	ret = Wifly_Set_TT_OK(PortNum, (char *)"set i a", IP, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set i n", Mask, 600);
	if (ret == TRUE)	ret = Wifly_Set_TT_OK(PortNum, (char *)"set i g", GW, 600);
	if (ret == TRUE)	ret = Wifly_Set_OK(PortNum, (char *)"set i d 0", 600);
	if (ret == TRUE) ret = Wifly_Save(PortNum);
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_FixIP", (char *)"FIXADA IP WLAN DEL WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Disconnect
**
** Descriptions:		Send 'leave' to disconnect the WLAN
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/

uint8_t Wifly_Disconnect(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char*)"leave", (char *)"DeAuth", 700);
#if LOG >= LOG_WARN
	Log_T_Print(LOG_WARN, ret, (char *)"Wifly_Disconnect", (char *)"DESCONNEXIO WLAN DEL WiFly");
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Ping
**
** Descriptions:		Send 'ping' with Address
**
** parameters:			UART port to send and Address to check ping
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Ping(uint8_t PortNum, char *Address)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_TT_Check(PortNum, (char *)"ping", Address, (char *)"PING", 3500);
#if LOG >= LOG_INFO
	char *string = NULL;
	string = strstr(Buffer, (char *)"PING");
	string = strtok(string, (char *)"\r\n");
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_Ping", (char *)"OPERACIO PING DEL WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_IP
**
** Descriptions:		Send 'get ip' to know the IP used in WLAN
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_IP(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"get ip", (char *)"IP=", 850);
#if LOG >= LOG_INFO
	char *string = NULL;
	string = strstr(Buffer, (char *)"IP=");
	string = strtok(string, (char *)":");
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_IP", (char *)"OBTENIR IP DEL WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_CheckIP
**
** Descriptions:		Send 'get ip' to know the IP used in WLAN and compare with sended IP
**
** parameters:			UART port to send and IP to compare
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_CheckIP(uint8_t PortNum, char *IP)
{
	uint8_t ret = FALSE;
	char *string = NULL;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"get ip", (char *)"IP=", 850);
	string = strstr(Buffer, (char *)"IP=");
	string = strtok(string, (char *)":");
	string = strtok(string, (char *)"=");
	string = strtok(NULL, (char *)"=");
	if (ret == TRUE)
	{
		if (strcmp(string, IP) == 0)	ret = TRUE;
		else	ret = FALSE;
	}
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_CheckIP", (char *)"COMPARAR IP DEL WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Channel
**
** Descriptions:		Send 'show net' to know the Channel used in WLAN
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Channel(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"show net", (char *)"Chan=", 850);
#if LOG >= LOG_INFO
	char *string = NULL;
	string = strstr(Buffer, (char *)"Chan=");
	string = strtok(string, (char *)"\r\n");
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_Channel", (char *)"OBTENIR CANAL WLAN WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_RSSI
**
** Descriptions:		Send 'show rssi' to know the signal RSSI in dBm
**
** parameters:			UART port to send
** Returned value:		RSSI Value or 0 if not found
**
*****************************************************************************/
int32_t Wifly_RSSI(uint8_t PortNum)
{
	int32_t ret = 0;
	char *string = NULL;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	if (Wifly_Set_Check(PortNum, (char *)"show rssi", (char *)"RSSI=", 850) == TRUE)
	{
		string = strstr(Buffer, (char *)"RSSI=");
		string = strtok(string, (char *)"\r\n");
		string = strstr(Buffer, (char *)"(");
		string = strtok(string, (char *)"()");
		ret = atoi(string);
	}
#if LOG == LOG_DBG
	int32_t cas = FALSE;
	if (ret != 0)	cas = TRUE;
	Log_TT_Print(LOG_DBG, cas, (char *)"Wifly_RSSI", (char *)"OBTENIR RSSI WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_UpTime
**
** Descriptions:		Send 'show time' to know the signal RSSI in dBm
**
** parameters:			UART port to send
** Returned value:		UpTime Value or 0 if not found
**
*****************************************************************************/
int32_t Wifly_UpTime(uint8_t PortNum)
{
	uint32_t ret = 0;
	char *string = NULL;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	if (Wifly_Set_Check(PortNum, (char *)"show time", (char *)"UpTime=", 850) == TRUE)
	{
		string = strstr(Buffer, (char *)"UpTime=");
		string = strtok(string, (char *)"\r\n");
		string = strstr(Buffer, (char *)"=");
		string = strtok(string, (char *)"= ");
		ret = atoi(string);
	}
#if LOG == LOG_DBG
	int32_t cas = FALSE;
	if (ret != 0)	cas = TRUE;
	Log_TT_Print(LOG_DBG, cas, (char *)"Wifly_UpTime", (char *)"OBTENIR UPTIME WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_GetPower
**
** Descriptions:		Send 'get wlan' to know the transmission Power
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_GetPower(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char *)"get wlan", (char *)"TxPower", 850);
#if LOG >= LOG_INFO
	char *string = NULL;
	string = strstr(Buffer, (char *)"TxPower");
	string = strtok(string, (char *)"\r\n");
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_GetPower", (char *)"OBTENIR POTENCIA WLAN WiFly", string);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_SetPower
**
** Descriptions:		Send 'set wlan tx XX' to change the transmission Power
** 						where XX is a range 0-12. Default 0 = 12 dBm (Max Value)
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_SetPower(uint8_t PortNum, uint32_t Value)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Print_Flush(PortNum);
	ret = Wifly_Set_TN_OK(PortNum, (char *)"set wlan tx", Value, 600);
#if LOG >= LOG_INFO
	Log_TN_Print(LOG_INFO, ret, (char *)"Wifly_SetPower", (char *)"CANVIAR POTENCIA WLAN WiFly", Value);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_ChangeBR
**
** Descriptions:		Change instantaneous Baudrate with 'set uart instant <value>'
**
** parameters:			UART port to send , new desired Baudrate
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/

uint8_t Wifly_ChangeBR(uint8_t PortNum, int32_t Value)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Print_Flush(PortNum);
	if (Wifly_Set_TN_OK(PortNum, (char *)"set u b", Value, 600) == TRUE);
	{
		Wifly_Save(PortNum);
		Wifly_Reboot(PortNum);
		Wifly_End(PortNum);
		Delay_Ms(DELAY,1000);
		ret = Wifly_Init(PortNum, Value);
		Print_Flush(PortNum);
		Estat = NORMAL;
	}
#if LOG >= LOG_INFO
	Log_TN_Print(LOG_INFO, ret, (char *)"Wifly_ChangeBR", (char *)"CANVIAR BAUDRATE WiFly", Value);
#endif
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_GET_Telnet
**
** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
**
** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_GET_Telnet(uint8_t PortNum, char *Host, char *Param)
{
	uint8_t ret = FALSE;
	char buf[SENDSIZE];
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	sprintf(buf, "%s %s", Host, (char *)NETPORT);
	Wifly_Open(PortNum, buf);
	sprintf(buf, "GET %s %s\r\nhost: %s\r\n\r\n", Param, (char *)WEB, Host);
	ret = Wifly_Set_Check(PortNum, buf, (char *)"200 OK", 6000);
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_GET_Telnet", (char *)"CONNEXIO INTERNET -> HTTP GET", Host);
#endif
	if (strstr(Buffer, (char *)"*CLOS*") == NULL)	Wifly_Close(PortNum);
	Wifly_Exit(PortNum);
	UnLock_Task(SEM_R1);
	free(buf);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_GET_Base
**
** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
**
** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_GET_Base(uint8_t PortNum, char *Host, char *Param)
{
	uint8_t ret = FALSE;
	char buf[SENDSIZE];
	Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Wifly_Set_OK(PortNum, (char *)"set i p 18", 600);
	Wifly_Set_TT_OK(PortNum, (char *)"set d n", Host, 600);
	Wifly_Set_OK(PortNum, (char *)"set i a 0", 600);
	Wifly_Set_TT_OK(PortNum, (char *)"set i r", HTTP, 600);
	sprintf(buf, "%s %s", Host, (char *)HTTP);
	Wifly_Open(PortNum, buf);
	sprintf(buf, "GET %s %s\r\nhost: %s\r\n\r\n", Param, (char *)WEB, Host);
	ret = Wifly_Set_Check(PortNum, buf, (char *)"200 OK", 6000);
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Wifly_GET_Base", (char *)"CONNEXIO INTERNET -> HTTP GET", Host);
#endif
	if (strstr(Buffer, (char *)"*CLOS*") == NULL)	Wifly_Close(PortNum);
	Wifly_Exit(PortNum);
	UnLock_Task(SEM_R1);
	free(buf);
	return ret;
}

/*****************************************************************************
** Function name:		Wifly_Web
**
** Descriptions:		Connect to specified Host and send string data, check response, close connection
**
** parameters:			UART port to send, HOST with Port to connect, Parameters to send
** 						and Buffer to receive data
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Wifly_Web(uint8_t PortNum, char *Host, char *Param, char **RecBuf)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R1);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	if (Wifly_Open(PortNum, Host) == TRUE)
	{
		Buffer[0] = '\0';
		ret = Wifly_Set_Check(PortNum, Param, (char *)"200 OK", 6000);
		*RecBuf = (char *)Buffer;
	}
	//UnLock_Task(SEM_R1);
	return ret;
}
